@@ -64,7 +64,8 @@ class TestAPYParams:
             "retry_timeout": "test",
             "observation_interval": "test",
             "drand_public_key": "test",
-            "history_duration": "test",
+            "history_interval_in_unix": 86400,
+            "history_start": 1652544875,
             "optimizer": {"timeout": param_value, "window_size": param_value},
             "testing": "test",
             "estimation": "test",
