@@ -26,11 +26,16 @@ from pathlib import Path
 from typing import Any, Dict, Union, cast
 from unittest import mock
 
-import atheris  # type: ignore
 import pytest
 from aea.exceptions import AEAEnforceError
 from aea.helpers.transaction.base import RawTransaction, SignedMessage
 
+
+try:
+    import atheris  # type: ignore
+except (ImportError, ModuleNotFoundError):
+    pytestmark = pytest.mark.skip
+
 from packages.open_aea.protocols.signing import SigningMessage
 from packages.valory.contracts.gnosis_safe.contract import (
     PUBLIC_ID as GNOSIS_SAFE_CONTRACT_ID,
